//
//  PlayerViewController.swift
//  AMPTVOSSample
//
//  Created by Cortes Saenz, David on 10/23/15.
//  Copyright © 2015 Akamai. All rights reserved.
//

import UIKit
import AVKit

import AmpCoreTV
import AmpMediaAnalyticsTV
import AmpOctoshapeTV
import AmpStatisticsTV
import AmpComScoreTV
import AmpAdobeHeartbeatTV
import AmpNielsenTV
import AmpIMATV
import InteractiveMediaAds

open class PlayerViewController: UIViewController, AVPlayerViewControllerDelegate, PlayerEventObserver, AmpOctoshapeObserver {
    
    open var ampPlayer: AmpPlayer!
    open var playerController: AVPlayerViewController? = AVPlayerViewController()
    open var ampMA: AmpMediaAnalyticsManager!
    open var ampOctoshape: AmpOctoshapeManager!
    open var ampStatistics: AmpStatisticsManager!
    open var ampComScore: AmpComScore!
    open var ampHeartbeat:AmpAdobeHeartbeat!
    open var ampNielsen: AmpNielsenManager!
    open var ampIMA: AmpIMAManager!
    
    var time: Double!
    var playerId: Int!
    
    let octoSourceTitle = "Octoshape Source:"
    let maBeaconTitle = "Media Analytics Beacon:"
    
    let imaPreMidPostRoll = "http://pubads.g.doubleclick.net/gampad/ads?sz=640x480&iu=/124319096/external/ad_rule_samples&ciu_szs=300x250&ad_rule=1&impl=s&gdfp_req=1&env=vp&output=vmap&unviewed_position_start=1&cust_params=deployment%3Ddevsite%26sample_ar%3Dpremidpost&cmsid=496&vid=short_onecue&correlator="
    
    override open func viewDidLoad() {
         super.viewDidLoad()
    }
    
    override open func pressesEnded(_ presses: Set<UIPress>, with event: UIPressesEvent?) {
        for item in presses {
            if item.type == .playPause {
                print("PlayPause")
            }
        }
    }
    
    func appCameBackFromBackground(_ notification : Notification) {
        self.ampPlayer.seekTo(self.time)
    }
    
    open func playerViewController(_ playerViewController: AVPlayerViewController, willResumePlaybackAfterUserNavigatedFrom oldTime: CMTime, to targetTime: CMTime) {
        self.ampMA?.notifyEndScrub()
    }
    
    open func playVideo(_ videoUrl: String, playerController: AVPlayerViewController) {
        
        self.ampPlayer = AmpPlayer()
        self.ampPlayer.autoplay = true
        
//        self.setupComScore()
        
//        self.setupNielsen()
        
        self.ampPlayer.setLicense("AwEm7pZvp53RWegXHgBO2rAnKwFGuEU7+3U9ro9U0Oog28jRBBPFAFBo/gymG5a41KmBfllPlxNymOuIgZeSmvs8NlVVitKzI+UfVq0d3h/u4pM2044a3tXXqQ5exrOJR6yoOuXkaO+aBo97g0nJ3SZ2VJURrryijEpgfyYW3Sa/EBmwfFMakZI+7sZ8ZaqgexyjLz5tL6bFu4m8uDmbAq8mSxtFQn6Yxp31LbEXRl49tQ==")
        
        print("Amp Version: \(self.ampPlayer.version)")
        
        self.setupPlayerController(playerController)
        
        // Register
        playerId = self.ampPlayer.registerObserver(self)
        
//        self.setupOctoshape()
        
        self.ampStatistics = AmpStatisticsManager(ampPlayer: self.ampPlayer, parentView: self.playerController!.view, containerFrame: self.playerController!.view.frame)

        self.setupHeartbeat()
        self.setupIMA(videoUrl: videoUrl)
        
        if (self.ampIMA == nil) {
            self.ampPlayer.handleUrl(videoUrl)
        }
        
//        self.setupAnalytics()
        
    }
    
    func setupOctoshape() {
        self.ampOctoshape = AmpOctoshapeManager(ampPlayer: self.ampPlayer, enableLogs: true, container:self.playerController!.view)
        self.ampOctoshape.registerObserver(self)
    }
    
    func setupIMA(videoUrl: String) {
        // IMATV: Instantiate the AMP IMA manager, it needs a reference to an `AmpPlayer`, and an optional fallback url
        //        to play in case of an error
        self.ampIMA = AmpIMAManager(ampPlayer: self.ampPlayer)
        self.ampIMA.playerViewController = self.playerController
        // IMATV: You can request ads immediately, to avoid further delay
        self.ampIMA.requestAds(contentSourceId: "19463", videoId: "googleio-highlights")
//        self.ampIMA.requestAds(contentSourceId: "2056", videoId: "5556532260001")
//        self.ampIMA.requestAds(assetKey: "58YYMTVcSYe-iz_BLsrb_g", suffixUrl: "hdnea=st=1433954196~exp=1907308086~acl=/*~hmac=62bfdfc61282e7be4d551454fd988dfafb20ec780d8011cc5c058b63d9b6f25d")
    }
    
    func setupHeartbeat() {
        let heartbeatConfig:[String:AnyObject] = ["trackingServer":"http://obumobile1.hb.omtrdc.net" as AnyObject, "channel":"test-channel" as AnyObject, "ovp":"test-ovp" as AnyObject, "videoId": "Required Video Id" as AnyObject, "videoName": "Non required video Name" as AnyObject, "debugLogging": true as AnyObject, "ssl": false as AnyObject]
        
        self.ampHeartbeat = AmpAdobeHeartbeat(ampPlayer: self.ampPlayer, configuration: heartbeatConfig)
        
//        let videoMetaData:[AmpMetaDataKey:AnyObject] = [AmpMetaDataKey.season: "Season 1" as AnyObject, AmpMetaDataKey.show: "Show 1" as AnyObject, AmpMetaDataKey.assetId:"Asset ID" as AnyObject]
        
        //        self.ampHeartbeat.setVideoMetaData(videoMetaData)
//        self.ampHeartbeat.setDebugLogging(value: true)
//        self.ampHeartbeat.setSSL(value: false)
        print("Adobe Heartbeat Video Version: \(ADBMediaHeartbeat.version())")
    }
    
    func setupComScore() {
        let labels = ["data": [
            "clientId": "0000000",
            "publisherSecret": "publisherSecretId",
            "appVersion": "0000000",
            "metadata": [
                "ads": [
                    "ns_st_ci": "#{media.guid}",
                    "c3": "*null",
                    "c4": "*null",
                    "c6": "*null",
                    "cb2": "*null",
                    "cb3": "*null",
                    "cb4": "*null",
                    "cb6": "*null"
                ],
                "video": [
                    "ns_st_ci": "#{media.guid}",
                    "c3": "*null",
                    "c4": "*null",
                    "c6": "*null",
                    "cb2": "*null",
                    "cb3": "*null",
                    "cb4": "*null",
                    "cb6": "*null"
                ]
            ]
            ]
        ]
        self.ampComScore = AmpComScore(publisherId: "1234567", publisherSecret: "9c455c81a801d3832a2cd281843dff30", ampPlayer: self.ampPlayer!, labels: labels)
    }
    
    func setupNielsen() {
        let appId = "T94C91A1F-C749-4847-ABB6-481365FD3C13"
        
        let appInfo = """
        {"appid": "\(appId)",
            "sfcode": "cert",
            "appname": "Akamai",
            "appversion":"3.0.0",
            "nol_devDebug": "DEBUG"}
        """
        
        self.ampNielsen = AmpNielsenManager(player: self.ampPlayer!, appInfo: appInfo)
        let metaData = """
            {"type": "content",
            "assetName": "myassetName",
            "length": "123",
            "title": "NiceName",
            "program": "myProgram",
            "assetid": "myAssetId",
            "channelName": "myChannel",
            "segB": "segmentB",
            "segC": "segmentC",
            "isfullepisode": "Y",
            "crossId1": "Reference11",
            "crossId2": "Reference22",
            "airdate": "20161013 20:00:00",
            "adloadtype": "2",
            "mediaURL": "http://www.akamai.com/",
            "segA":"segmentA"}
        """
        self.ampNielsen?.set(metaData: metaData)
    }
    
    func setupAnalytics() {
        let analyticsUrl = "https://ma1019-r.analytics.edgekey.net/config/beacon-8566.xml"
        
        let customData = [
            "title": "AKaMTitle",
            "category": "AKamCategory",
            "playerId": "AKAM AVPlayer playerId",
            "device": "AKAM AVPlayer device",
            "show": "AKAM AVPlayer$show"
        ]
        
        self.ampMA = AmpMediaAnalyticsManager(ampPlayer: self.ampPlayer!, configUrl: analyticsUrl, customDimensions: customData as [String : AnyObject])
        
        self.ampStatistics.addSegment("Media Analytics Beacon:", info: self.ampMA.configUrl!)
    }
    
    func setupPlayerController(_ playerController: AVPlayerViewController) {
        self.playerController = playerController
        
        self.playerController?.delegate = self
        self.playerController?.player = self.ampPlayer.player
        self.playerController?.view.frame = self.view.frame
        self.addChildViewController(self.playerController!)
        self.view.addSubview(self.playerController!.view)
    }
    
    override open func didReceiveMemoryWarning() {
        // super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    open func onPlaybackEnded(_ ampPlayer: AmpPlayer) {
        print("onPlaybackEnded")
    }
    
    open func onPlaybackStateChanged(_ ampPlayer: AmpPlayer) {
        if (ampPlayer.playbackState == .ended) {
            print("ampPlayer.playbackState == .ended")
        }
    }
    
    open func onBufferingStateChanged(_ ampPlayer: AmpPlayer) {
        if (ampPlayer.bufferingState == BufferingState.ready) {
            print("BufferingState.Ready")
        }
    }
    
    open func onAmpError(_ ampPlayer: AmpPlayer, error: PlayerError) {
        switch(error) {
        case PlayerError.unknownAssetFormat:
            print("PlayerError.UnknownAssetFormat")
        case PlayerError.downloadFailed:
            print("PlayerError.DownloadFailed")
        case PlayerError.unknownError:
            print("PlayerError.UnknownError")
        }
    }
    
    open func onHandledResignActive(_ ampPlayer: AmpPlayer) {
        ampPlayer.pause()
    }
    
    open func onPositionChange(_ ampPlayer: AmpPlayer) {
        //        print("time: %d", ampPlayer.currentTime())
    }
    
    open func onCuepointReached(_ ampPlayer: AmpPlayer, atTime: Double) {
        print("cuepoint: %d - time: %d", atTime, ampPlayer.currentTime())
    }
    
    open func onCurrentStreamInfoEvent(_ currentStream: OCCurrentStreamInfoEvent) {
        print("CurrentStreamInfo - bitrate: %d", currentStream.rateset)
        self.ampStatistics.updateSegment(AmpStatisticsManager.bitrateTitle, info: AmpStatisticsManager.formatBitrate(Double(currentStream.rateset)))
        
    }
    
    open func onStreamInformationChanged(_ ampPlayer: AmpPlayer, streamInfo: StreamInfo) {
        
    }
    
    open func onMultiStreamInfoEvent(_ multiStream: OCMultiStreamInfoEvent) {
        multiStream.baseStreams.map({stream in stream as! NSDictionary})
            .forEach { print("Stream:%@ - bitrate: %@, channel: %@", String(describing: $0), $0["bitrate"], $0["name"]) }
    }
    
    open func onStreamSourceInfoEvent(_ streamSource: OCStreamSourceInfoEvent) {
        if let streamSource = streamSource.streamSource {
            print("Source: ", streamSource)
            if self.ampStatistics.hasSegment(octoSourceTitle) {
                self.ampStatistics.updateSegment(octoSourceTitle, info: streamSource)
            } else {
                self.ampStatistics.addSegment(octoSourceTitle, info: streamSource)
            }
        }
        
        if let multiCastStatus = streamSource.multiCastStatus {
            print("MultiCast Status: ", multiCastStatus)
        }
    }
    
    // When we go back to collection view
    override open func viewWillDisappear(_ animated:Bool) {
        // We need to ignore the pause event the playerViewController sends when we assign the player to nil, otherwise a pause event will be sent to all analytics providers
        self.ampPlayer.ignorePause = true
        self.playerController?.delegate = nil
        self.playerController?.player = nil
        self.playerController?.view?.removeFromSuperview()
        self.playerController?.removeFromParentViewController()
        self.playerController = nil
        
        self.ampPlayer.stop()
        self.ampPlayer = nil
        print("Stopped")
        super.viewWillDisappear(animated)
    }
    
}

// IMATV: Extend the IMAEventObserver, for more events check the protocol's definition
extension PlayerViewController: IMAEventObserver {
    
    // IMATV: If any error is recevied by IMA, you'll receive it below as well
    public func onIMAError(_ imaManager: AmpIMAManager, error: Error) {
        print("IMA ERROR \(error)")
    }
    
    public func onIMAAdStarted(_ imaManager: AmpIMAManager, adBreakInfo: IMAAdBreakInfo) {
        print("Ad Started")
    }
    
    public func onIMAAdEnded(_ imaManager: AmpIMAManager, adBreakInfo: IMAAdBreakInfo) {
        print("Ad Ended")
    }
    
}

